@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Author:   $Author: merkosh $
-# Revision: $Rev: 162 $
+# Revision: $Rev: 214 $
 ############################################################################
 #    Copyright (C) 2005 by Uwe Mayer                                       #
 #    merkosh@hadiko.de                                                     #
@@ -25,7 +25,7 @@
 #-- Qt imports
 from qt import QImage, QPixmap, QLabel, QGridLayout, PYSIGNAL
 from qt import QImageDrag, QDragObject, QSizePolicy, QPopupMenu, QIconSet
-from qt import QCursor, QPixmap, QFileDialog, QMessageBox
+from qt import QCursor, QPixmap, QFileDialog, QMessageBox, QFileDialog
 
 
 #-- Python imports
@@ -232,6 +232,89 @@ saveAs_data = \
     "\xa5\xaa\x00\x00\x00\x00\x49\x45\x4e\x44\xae\x42" \
     "\x60\x82"
 
+
+link_data = \
+    "\x89\x50\x4e\x47\x0d\x0a\x1a\x0a\x00\x00\x00\x0d" \
+    "\x49\x48\x44\x52\x00\x00\x00\x0f\x00\x00\x00\x0f" \
+    "\x08\x06\x00\x00\x00\x3b\xd6\x95\x4a\x00\x00\x01" \
+    "\xb5\x49\x44\x41\x54\x28\x91\x7d\x93\xb1\x6b\xdb" \
+    "\x40\x14\xc6\x7f\x17\x6e\xd0\x90\xa1\x1e\x05\x19" \
+    "\x9a\x31\xd0\xc5\xdc\x14\x2b\xd5\x90\x40\x21\x19" \
+    "\x4d\x3a\xd4\x4b\x49\xf0\x12\x93\xe0\xc1\x74\x08" \
+    "\x9a\x33\x79\x32\x88\x94\x80\x9a\xd2\x45\x8b\xfd" \
+    "\x17\x04\xd2\xc1\x70\x0a\x04\x0e\x0f\x1d\xb2\x74" \
+    "\xe8\xe6\xb5\xe3\x0d\x82\xeb\xa0\x48\x1c\xae\x93" \
+    "\x5b\x1e\x8f\x77\xdf\xf7\xde\xfb\xbe\x3b\x61\x1e" \
+    "\x0d\x94\xa0\x62\xe5\xf0\x8e\x29\x8c\xb0\xa5\x25" \
+    "\x90\x01\x94\x80\x84\xd5\x5c\xda\xd2\x12\xc5\x91" \
+    "\x4b\x27\x29\x01\x01\xcb\xbf\x4b\xc2\x37\x21\xaa" \
+    "\xa3\x9c\x4f\xb0\x2e\x0a\x24\x2e\x9d\xa4\xec\xb6" \
+    "\x77\x51\xb1\x02\x20\xf9\x92\xd0\x7e\xd7\xe6\xf8" \
+    "\xf3\xf1\xab\x13\x48\xa0\x01\xd6\x17\xa3\x38\x72" \
+    "\x83\xb3\x01\xb3\x6f\xb3\x57\x27\x90\x40\xc5\x48" \
+    "\xc5\x78\xf0\xe1\xc0\x01\x5c\x7f\xbd\xc6\x9e\x58" \
+    "\xea\xba\x0f\xac\x73\x09\xb0\xf8\xb5\x60\x7c\x35" \
+    "\x26\x8a\x23\x07\xd0\xfb\xd8\x03\x09\xb7\xdf\x6f" \
+    "\x2b\xb0\x37\xaa\x1f\x37\x4c\x61\x44\x7f\xd8\x67" \
+    "\xff\xfd\x3e\xc9\x65\xd2\xa8\xbd\xf3\x76\x87\xec" \
+    "\x26\x03\x40\x75\x94\x8b\xe2\xc8\xa9\x8e\x72\x3e" \
+    "\xc1\x86\x2d\x2d\xa6\x30\x42\xc5\x8a\xee\x61\x97" \
+    "\xd1\xc5\x88\x7c\x9a\x13\x6e\x85\x6c\x87\xdb\xa4" \
+    "\x93\x14\x80\xd1\xc5\x08\x33\x37\xf8\x04\xa2\xf6" \
+    "\x19\x59\x75\xf0\xbd\xce\x6e\x32\x96\x7f\x96\x3c" \
+    "\xfd\x7e\x22\x9f\xe6\x24\x97\x09\xdd\xc3\x6e\x23" \
+    "\xae\xd0\x85\x5e\xeb\x63\x4d\xd4\xfb\xd4\x83\x12" \
+    "\xf2\x69\x0e\xc0\xf8\x6a\x4c\x6b\xb3\x45\x7f\xd8" \
+    "\x47\xae\x53\xd1\x96\x95\xca\xa7\x27\xa7\x8d\x68" \
+    "\x41\x10\xa0\xef\xb4\x50\xb1\x72\xfa\xa7\x06\xa8" \
+    "\x76\x6e\x00\xfe\x13\x04\x8e\xf6\x8e\x18\x9c\x0d" \
+    "\x00\xb8\xbf\xbb\x17\xcf\xc6\x36\x75\xb9\xda\xb1" \
+    "\xce\x4d\x61\x84\xea\x28\x37\xfb\x31\xa3\xb5\xd9" \
+    "\x6a\x6c\x34\x73\xc3\xc3\xe2\x01\x80\x17\x77\xf6" \
+    "\x77\xcf\x26\x59\xf3\xe6\x2d\x96\xf3\xe1\x39\x7a" \
+    "\xae\x85\x30\x8f\xe6\xbf\xdf\xb2\x9a\xaf\xba\xa0" \
+    "\xe7\x5a\x04\x32\xe0\x1f\x03\xed\x10\x8b\x0c\x60" \
+    "\x59\x7c\x00\x00\x00\x00\x49\x45\x4e\x44\xae\x42" \
+    "\x60\x82"
+
+embed_data = \
+    "\x89\x50\x4e\x47\x0d\x0a\x1a\x0a\x00\x00\x00\x0d" \
+    "\x49\x48\x44\x52\x00\x00\x00\x0f\x00\x00\x00\x0f" \
+    "\x08\x06\x00\x00\x00\x3b\xd6\x95\x4a\x00\x00\x01" \
+    "\x75\x49\x44\x41\x54\x28\x91\x8d\x91\x31\x6b\x13" \
+    "\x71\x18\x87\x9f\x03\x87\x0b\x9c\x83\x83\xfb\x05" \
+    "\xe2\xd0\xc1\x0f\x70\x25\x42\xcf\xc1\x41\xb7\x8c" \
+    "\x19\xed\x57\x48\xa7\xae\x7e\x85\x6e\x8e\xba\x88" \
+    "\x2e\x72\x38\xd4\xc4\x82\x72\x77\x20\xb4\x82\x52" \
+    "\x97\x40\x0f\x14\x2c\x64\xe8\xd0\x82\x29\x59\x1e" \
+    "\x87\xbb\xb4\x69\x9a\x16\xff\xcb\xc3\xfb\xc2\xcb" \
+    "\xf3\x7b\xdf\x3f\x2a\x16\xa2\x92\x17\x39\xcb\x75" \
+    "\x18\x85\x86\x51\xe8\xaa\xfa\x62\x60\x15\xc3\x28" \
+    "\x34\x33\x53\x75\x3e\x94\x99\xc9\x0c\xc3\x28\xbc" \
+    "\xd9\x38\xaf\xc3\x28\x34\xb6\x27\x22\x47\xc8\x4f" \
+    "\xfe\xcf\xbc\x98\x20\xb6\x27\x3b\x5c\x59\x21\x50" \
+    "\x69\x3f\x68\xcb\xd2\xab\xc6\x55\x50\x94\x05\xeb" \
+    "\xc9\x3a\x94\xd0\x7a\xd2\x12\x60\x7a\x36\x0d\x28" \
+    "\x81\x04\x88\x3b\xb1\xaa\x9e\x1e\xd6\x3c\xd9\x57" \
+    "\xb5\xe9\xdf\x9a\xe8\x0e\x00\x67\x07\x70\xb7\x0d" \
+    "\x3f\x36\xe1\xe1\x0b\xf8\xb6\x49\x35\xae\x56\x26" \
+    "\x5a\x4c\x76\x69\x3e\x78\xae\xf6\xf5\x6b\xbf\xe6" \
+    "\xf7\xed\xba\x3f\x19\x35\xcc\x2e\x18\x77\x62\xf3" \
+    "\x22\x6f\x62\x9f\xa8\x1e\x69\xd1\x57\x0f\xf5\x73" \
+    "\xaf\xe1\x53\x75\x5f\x3f\xa5\x3a\x4b\x75\x37\xbd" \
+    "\xb2\x52\x1d\xfb\x1e\x50\x0e\xea\x23\x7c\xd9\x82" \
+    "\x47\xc0\xde\x16\xa4\x53\xf8\x38\x80\x0d\x60\x78" \
+    "\x5e\xf3\xf8\x1d\x00\x45\x59\x34\xe6\xc9\xa8\x31" \
+    "\x34\xa6\xdd\x54\x67\xb9\x7e\x48\xf5\x6f\xa2\xef" \
+    "\x13\x3d\x1d\xd5\xbc\x66\xbe\xff\x18\xf6\x9e\x41" \
+    "\xb7\x31\x75\xcf\x61\x38\x80\x6e\x63\xec\xc2\x74" \
+    "\x38\xa0\xb5\x01\xfc\x7e\xbd\x6c\xce\x96\x8c\xc9" \
+    "\x82\x31\xd3\x37\x6b\x3a\x59\xab\xb9\x60\xbe\xbc" \
+    "\xf6\x9f\xb7\x0d\x5f\xd5\xfc\x35\xe7\xcb\x6b\x9c" \
+    "\x5f\x3b\x88\x3b\xf1\x8d\x7f\x79\xdb\xab\xc6\x55" \
+    "\xf0\x0f\x9d\x06\xe0\x43\x91\x8b\xbb\x55\x00\x00" \
+    "\x00\x00\x49\x45\x4e\x44\xae\x42\x60\x82"
+
 #-- preferences ----------------------------------------------------------------
 pref = getPreferences(__name__)
 
@@ -295,6 +378,10 @@ class PicturePreview(PicturePreviewBase):
         self.paste_pix.loadFromData(paste_data, "PNG")
         self.saveAs_pix = QPixmap()
         self.saveAs_pix.loadFromData(saveAs_data, "PNG")
+        self.link_pix = QPixmap()
+        self.link_pix.loadFromData(link_data, "PNG")
+        self.embed_pix = QPixmap()
+        self.embed_pix.loadFromData(embed_data, "PNG")
         
         gridLayoutLabel = QGridLayout( self, 1,1 )
         gridLayoutLabel.addWidget( self.labelImage, 0,0 )
@@ -315,21 +402,21 @@ class PicturePreview(PicturePreviewBase):
         # only accept left mouse button
         if (e.button() == e.RightButton):
             menu = QPopupMenu(self)
-            menu.insertItem(self.tr("Picture Menu"), None)
+            menu.insertItem(self.tr("Picture Menu"), None, 1)
             menu.insertSeparator()
-            if (self.img): menu.insertItem(QIconSet(self.copy_pix), self.tr("Copy"), self.copyPicture)
-            menu.insertItem(QIconSet(self.paste_pix), self.tr("Paste"), self.pastePicture)
-            if (self.img): menu.insertItem(QIconSet(self.saveAs_pix), self.tr("Save As"), self.saveAsPicture)
-            if (self.img): menu.insertItem(QIconSet(self.recycle_pix), self.tr("Delete"), self.deletePicture)
+            menu.insertItem(QIconSet(self.link_pix), self.tr("Link picture"), self.linkPicture, 0, 2)
+            menu.insertItem(QIconSet(self.embed_pix), self.tr("Embedd picture"), self.embedPicture, 0, 3)
+            menu.insertSeparator()
+            if (self.img): menu.insertItem(QIconSet(self.copy_pix), self.tr("Copy"), self.copyPicture, 0, 4)
+            menu.insertItem(QIconSet(self.paste_pix), self.tr("Paste"), self.pastePicture, 0, 5)
+            if (self.img): menu.insertItem(QIconSet(self.saveAs_pix), self.tr("Save As"), self.saveAsPicture, 0, 6)
+            if (self.img): menu.insertItem(QIconSet(self.recycle_pix), self.tr("Delete"), self.deletePicture, 0, 7)
 
-            if (self.img):
-                if (self.copyImg == None):
-                    menu.setItemEnabled(menu.idAt(3), False)
-                if (self.img == None):
-                    menu.setItemEnabled(menu.idAt(4), False)
-            else:
-                if (self.copyImg == None):
-                    menu.setItemEnabled(menu.idAt(2), False)
+            # grey-out invalid fields
+            if (self.img == None):
+                menu.setItemEnabled(4, False)
+            if (self.copyImg == None):
+                menu.setItemEnabled(5, False)
                     
             menu.exec_loop(QCursor.pos())
             return
@@ -340,6 +427,58 @@ class PicturePreview(PicturePreviewBase):
             dlg.exec_loop()
 
 
+    def linkPicture(self):
+        """links a picture on the file system"""
+        # get input filename
+        filename = QFileDialog.getOpenFileName(
+            pref['defaultPath'], self.tr("Images (*.bmp; *.jpeg; *.pbm; *.png; *.ppm *.xbm; *.xpm)"),
+            self, None, self.tr("Select image"))
+        # exit on abort
+        if (filename == None): return
+        # converto to non QString object
+        filename = unicode(filename)
+
+        if (self._loadImage(filename)):
+            # try to remove absolute portion of path - if prefix is <basedir>
+            if (filename[:len(self.basedir)] == self.basedir):
+                self.log.info("Converting absolute to relative path")
+                filename = filename[len(self.basedir)+1:]
+                self.log.info("Linking to file %s", filename)
+            else:
+                self.log.info("Linking to absolute path %s", filename)
+
+            # set record fields
+            self.record['embeddedImage'] = ""
+            self.record['picture'] = unicode(filename)
+
+        # notify of modification
+        self.emit(PYSIGNAL("modified"), ())
+        # update display
+        self.resizeImage()
+
+
+    def embedPicture(self):
+        """embeds a picture in the record"""
+        # get input filename
+        filename = QFileDialog.getOpenFileName(
+            pref['defaultPath'], self.tr("Images (*.bmp; *.jpeg; *.pbm; *.png; *.ppm *.xbm; *.xpm)"),
+            self, None, self.tr("Select image"))
+        # exit on abort
+        if (filename == None): return
+        # converto to non QString object
+        filename = unicode(filename)
+        self.log.info("Embedding image file %s", filename)
+
+        # load picture
+        if (self._loadImage(filename)):
+            self._embedImage()
+
+        # notify of modification
+        self.emit(PYSIGNAL("modified"), ())
+        # update display
+        self.resizeImage()
+        
+
     def saveAsPicture(self):
         """saves the picture as a file"""
         # skip out when no picture available
@@ -349,7 +488,7 @@ class PicturePreview(PicturePreviewBase):
         
         # display dialog
         filename = QFileDialog.getSaveFileName(
-            pref['defaultPath'], self.tr("Images (*.bmp, *.jpeg, *.pbm, *.png, *.ppm *.xbm, *.xpm)"),
+            pref['defaultPath'], self.tr("Images (*.bmp; *.jpeg; *.pbm; *.png; *.ppm *.xbm; *.xpm)"),
             self, None, self.tr("Save file as"))
         # exit on abort
         if (filename == None): return
@@ -371,7 +510,7 @@ class PicturePreview(PicturePreviewBase):
             if (ans == QMessageBox.No): return
 
         # save file
-        self.log.info("Saving image as %s", basename(filename))        
+        self.log.info("Saving image as %s", filename)
         ok = self.img.save(filename, str(imgType))
         if (not ok):
             self.log.warning("Could not save picture to file")
@@ -399,7 +538,7 @@ class PicturePreview(PicturePreviewBase):
                     self.log.warning("Not removing image from filesystem: file not found")
 
         # decode and display changes
-        self.imageDecode()              # decode to empty picture
+        self.defaultImageDecode()       # decode to empty picture
         self.resizeImage()
         
 
@@ -420,9 +559,9 @@ class PicturePreview(PicturePreviewBase):
     def onImageDropped(self, img):
         """called when image dropped"""
         self.img = img
-        self.record.modified(True)      # causes record to be saved
-        self.imageDecode()              # update record
-        self.resizeImage()              # update on screen
+        self.defaultImageDecode()       # update record
+        # write access to the record automatically sets dirty flag
+        # self.record.modified(True)      # causes record to be saved
 
 
     #-- interface --------------------------------------------------------------
@@ -450,40 +589,131 @@ class PicturePreview(PicturePreviewBase):
         return self.record
 
 
-    def imageDecode(self):
-        """decodes internal image to be saved in the record"""
-        #-- handle embedded image
-        if (pref['embeddedImage']):
-            # extract image from QImage 
-            data = IODeviceBuffer()
-            
-            if (self.img == None):
-                self.log.debug("Saving empty image")
-                self.record['embeddedImage'] = ""
-                self.record['picture'] = u""                
-            elif ( self.img.save(data, pref['imageFormat']) ):
-                self.log.debug("Saving embedded image: %d bytes", data.size())
-                self.record['embeddedImage'] = data.readAll()
-                self.record['picture'] = ".%s"%pref['imageFormat'].lower()
-                data.close()            # free memory
+    def _loadImage(self, filename):
+        """loads an image from external reference
+
+        Return: True -- loading successfull
+                        self.img is set to the loaded image
+                False -- loading unsuccessfull
+                         self.img is set to None
+                         record data is cleared
+        """
+        self.img = None
+
+        # load picture
+        f = openResource(filename)
+        if (f != None):
+            self.img = QImage()
+            ok = self.img.loadFromData(f.read())
+            f.close()
+            if (not ok):
+                self.log.error("Could not identify image data")
+                self.img = None
+                self.record['embeddedImage'] = u""
+                self.record['picture'] = u""
+                return False
+
             else:
-                self.log.error("Could not save image to record field")
+                # ok: self.img already set to QImage instance
+                return True            
+        else:
+            # failed: self.img already set to None
+            return False
+
+
+    def _embedImage(self):
+        """decodes internal image to be saved embedded in the record"""
+        # extract image from QImage 
+        data = IODeviceBuffer()
+            
+        if (self.img == None):
+            self.log.debug("Saving empty image")
+            self.record['embeddedImage'] = ""
+            self.record['picture'] = u""                
+        elif ( self.img.save(data, pref['imageFormat']) ):
+            self.log.debug("Saving embedded image: %d bytes", data.size())
+            self.record['embeddedImage'] = data.readAll()
+            fmt = pref['imageFormat']
+            if (fmt == 'JPEG'): fmt = 'JPG' # AMC requires 'jpg' instead of 'jpeg'
+            self.record['picture'] = "."+fmt.lower() # must be ASCII string
+            data.close()            # free memory
+        else:
+            self.log.error("Could not save image to record field")
+            self.record['embeddedImage'] = ""
+            self.record['picture'] = u""
+            
+
+    def _saveLinkedImage(self, filename):
+        """decodes internal image to be saved linked in the record"""
+        # copy to target buffer is successfull
+        if (self.img == None):
+            self.log.info("Saving empty image")
+            self.record['embeddedImage'] = ""
+            self.record['picture'] = u""                              
+            return
+
+        # determine and correct file format
+        fmt = splitext(filename)[1][1:].upper()
+        if (fmt == 'JPG'): fmt = 'JPEG'
+        if (not fmt in ['JPEG', 'BMP', 'PBM', 'PGM', 'PNG', 'PPM', 'XBM', 'XPM']):
+            self.log.warning("Format '%s' unknown; falling back to default: JPEG", fmt)
+            fmt = 'JPEG'
+            filename = splitext(filename)[0]+".jpeg"
+
+        # copy data to target file
+        data = IODeviceBuffer()
+
+        if (self.img.save(data, str(fmt))):
+            self.log.debug("Saving linked image %s: %d bytes", filename, data.size())
+            try:
+                f = open(filename, "wb")
+                f.write(data.readAll())
+                f.close
+                data.close()        # free memory
+            except IOError, e:
+                self.log.error("Could not write data to disk: %s", unicode(e))
                 self.record['embeddedImage'] = ""
                 self.record['picture'] = u""
-
-        #-- handle save image as link
+            else:
+                # try to remove absolute portion of path - if prefix is <pathToPicture>
+                if (filename[:len(self.basedir)] == self.basedir):
+                    self.log.info("Converting absolute to relative path")
+                    filename = filename[len(self.basedir)+1:]
+                else:
+                    self.log.info("Not converting filename to relative path")
+                
+                self.record['embeddedImage'] = ""               
+                self.record['picture'] = filename                
+        # could not save data to target buffer
         else:
-            pathToPicture = self.basedir # assume relative path
+            self.log.error("Could not save image to intermediate buffer")
+            self.record['embeddedImage'] = ""
+            self.record['picture'] = u""
+
+
+    def defaultImageDecode(self):
+        """decodes internal image to be saved in the record"""
+        # handle embedded image
+        if (pref['embeddedImage']):
+            self._embedImage()
             
+        # handle save image as link
+        else:
+            #-- generate filename template
+            pathToPicture = self.basedir # assume relative path
+
             # absolute picture path or linked image
             if ((pref['filenamePrefix'][0] in ('/', )) or
                 (pref['filenamePrefix'][:7] in ('http://',))):
                 pathToPicture = ""
 
-            # build filename from prefix (including suffix)
+            # build filename from configuration prefix (including suffix)
+            fields = {'no': self.record['number'],
+                      'db': splitext(basename(pref['recentFiles'][0]))[0],
+                      }
             try:
-                filename = (pref['filenamePrefix']%self.record['number'])+\
-                           ".%s"%pref['imageFormat'].lower()
+                filename = pref['filenamePrefix']%fields+\
+                           "."+pref['imageFormat'].lower()
             except TypeError, e:
                 self.log.error("Invalid filename prefix: %s", unicode(e))
                 self.record['embeddedImage'] = ""
@@ -491,50 +721,42 @@ class PicturePreview(PicturePreviewBase):
 
             # create target directory
             abs_filename = fjoin(pathToPicture, filename)
-            filedir = dirname(abs_filename)
-            if (not fexists(filedir)):
-                self.log.info("Creating target directory %s", filedir)
-                try:
-                    os.makedirs(filedir)
-                except IOError, e:
-                    self.log.error("Could not create target directory: %s", unicode(e))
-                    self.record['embeddedImage'] = ""
-                    self.record['picture'] = u""
+
+            #-- query for filename directly
+            if (pref['queryPathOnLinkedImage']):
+                # get input filename
+                abs_filename = QFileDialog.getSaveFileName(
+                    abs_filename, self.tr("Images (*.bmp; *.jpeg; *.pbm; *.png; *.ppm *.xbm; *.xpm)"),
+                    self, None, self.tr("Select target filename"))
+                # exit on abort
+                if (abs_filename == None): return
+                # convert to non QString object
+                abs_filename = unicode(abs_filename)
+                # **directory has to exist** - but file may already exist
+
+            #-- use generated filename
+            else:
+                filedir = dirname(abs_filename)
+                if (not fexists(filedir)):
+                    self.log.info("Creating target directory %s", filedir)
+                    try:
+                        os.makedirs(filedir)
+                    except IOError, e:
+                        self.log.error("Could not create target directory: %s", unicode(e))
+                        self.record['embeddedImage'] = ""
+                        self.record['picture'] = u""
+                        return
 
             # test if file exists
             if (fexists(abs_filename)):
                 self.log.warning("Target file %s already exists: overwriting", abs_filename)
 
-            # copy data to target file
-            data = IODeviceBuffer()
-            # copy to target buffer is successfull
-            if (self.img == None):
-                self.log.info("Saving empty image")
-                self.record['embeddedImage'] = ""
-                self.record['picture'] = u""                              
-                
-            elif (self.img.save(data, pref['imageFormat'])):
-                self.log.debug("Saving linked image %s: %d bytes", abs_filename, data.size())
-                try:
-                    f = open(abs_filename, "wb")
-                    f.write(data.readAll())
-                    f.close
-                    data.close()        # free memory
-                except IOError, e:
-                    self.log.error("Could not write data to disk: %s", unicode(e))
-                    self.record['embeddedImage'] = ""
-                    self.record['picture'] = u""
-                else:
-                    self.record['embeddedImage'] = str(splitext(filename)[1]) # MUST be ASCII string
-                    self.record['picture'] = filename                
-            # could not save data to target buffer
-            else:
-                self.log.error("Could not save image to intermediate buffer")
-                self.record['embeddedImage'] = ""
-                self.record['picture'] = u""
+            self._saveLinkedImage(abs_filename)
 
         # notify of modification
         self.emit(PYSIGNAL("modified"), ())
+        # update on screen
+        self.resizeImage()              
         
         
     def setRecord(self, rec):
@@ -543,7 +765,7 @@ class PicturePreview(PicturePreviewBase):
         # remember record
         self.record = rec
 
-        #-- read record fields and put picture, if available on streen
+        #-- read record fields and put picture, if available on screen
         if (self.record == None):
             self.img = None
             
@@ -568,22 +790,15 @@ class PicturePreview(PicturePreviewBase):
                 pathToPicture = ""
                 
             # load picture
-            f = openResource(fjoin(pathToPicture, self.record['picture']))
-            if (f != None):
-                self.img = QImage()
-                ok = self.img.loadFromData(f.read())
-                f.close()
-                if (not ok):
-                    self.log.error("Could not identify embedded image data")
-                    self.img = None
-            # could not access file
-            else:
-                self.img = None
-                
+            self._loadImage(fjoin(pathToPicture, self.record['picture']))            
+               
         # new record: display no picture
         else:
             self.img = None
 
+        # notify of modification
+        self.emit(PYSIGNAL("modified"), ())
+
         # at this point the image can be:
         # - from http (or whatever openResource loads)
         # - from (correctly) linked image
@@ -596,8 +811,6 @@ class PicturePreview(PicturePreviewBase):
         self.resizeImage()
 
 
-
-
 #-- testroutine ----------------------------------------------------------------
 if (__name__ == "__main__"):
     from qt import QApplication, QObject, SIGNAL, SLOT
