@@ -27,9 +27,9 @@ from os.path import join as fjoin, exists as fexists
 pythondir = "${prefix}/lib/python2.4/site-packages".replace('${prefix}', "/usr/local")
 installdir = fjoin(pythondir, "lmc")
 
-if (os.getcwd() == "/home/merkosh/lmc-0.12.2/src"):
+if (os.getcwd() == "/home/merkosh/Projekte/lmc/src"):
 	installdir = "."
 
-version = "0.12.2"
+version = "0.12.3"
 scriptdir = fjoin(installdir, "scripts")
 resourcedir = fjoin(installdir, "resources")
